@@ -1,9 +1,12 @@
-;;; mhc.el --- MH Calendar.
+;;; mhc.el --- Message Harmonized Calendaring system.
 
+;; Description: Message Harmonized Calendaring system.
 ;; Author:  Yoshinari Nomura <nom@quickhack.net>
-;;
-;; Created: 1994/07/04
-;; Revised: $Date: 2009/05/31 12:54:50 $
+;; Created: 1994-07-04
+;; Version: 1.1.0
+;; Keywords: calendar
+;; URL: http://www.quickhack.net/mhc
+;; Package-Requires: ((calfw "20150703"))
 
 ;;;
 ;;; Commentary:
@@ -487,10 +490,6 @@ If HIDE-PRIVATE, private schedules are suppressed."
 
 (defvar mhc-face-week-color-paint-thick nil)
 
-(defvar mhc-summary-buffer-current-date-month nil
-  "Indicate summary buffer's month. It is also used by mhc-summary-buffer-p")
-(make-variable-buffer-local 'mhc-summary-buffer-current-date-month)
-
 (defun mhc-expand-date-scope-backward (date scope)
   "Expand date scope backward involving the whole first week of month.
 DATE can be any date of the target month.
@@ -572,6 +571,14 @@ If SUBJECT-ONLY is non-nil, it will search only on X-SC-Subject:"
         (message "No match")
       (mhc-scan match))))
 
+(defun mhc-search-recurrence (recurrence-tag)
+  "Search events by RECURRENCE-TAG."
+  (interactive "sSearch recurrence-tag: ")
+  (let* ((match (mhc-db-search :recurrence_tag recurrence-tag)))
+    (if (null match)
+        (message "No match")
+      (mhc-scan match))))
+
 (defun mhc-scan (events &optional insert-current-buffer clip-from clip-to)
   "Create mhc-summary buffer using EVENTS list.
 If INSERT-CURRENT-BUFFER is non-nil, insert contents in the current buffer.
@@ -607,7 +614,7 @@ Returns t if the importation was succeeded."
        (list (get-buffer (read-buffer "Import buffer: "
                                       (current-buffer))))))
   (let ((draft-buffer (generate-new-buffer mhc-draft-buffer-name))
-        (current-date (or (mhc-current-date) (mhc-calendar-get-date) (mhc-date-now)))
+        (current-date (or (mhc-summary-current-date) (mhc-calendar-get-date) (mhc-date-now)))
         (succeed t)
         msgp date time subject location category recurrence-tag priority alarm)
     (and (called-interactively-p 'interactive)
@@ -632,7 +639,7 @@ Returns t if the importation was succeeded."
     (condition-case ()
         (if import-buffer
             (progn
-              (delete-other-windows)
+              (delete-other-windows-vertically)
               (goto-char (point-min))
               (if (y-or-n-p "Do you want to import this article? ")
                   (let* ((original (with-current-buffer
@@ -762,13 +769,7 @@ Returns t if the importation was succeeded."
                      (mhc-date-format day "%04d%02d%02d" yy mm dd))
                    date " ")
                   "\nX-SC-Time: "
-                  (if time
-                      (let ((begin (car time))
-                            (end (nth 1 time)))
-                        (concat
-                         (if begin (mhc-time-to-string begin) "")
-                         (if end (concat "-" (mhc-time-to-string end)) "")))
-                    "")
+                  (mhc-time-range-to-string time)
                   "\nX-SC-Category: "
                   (mapconcat (function capitalize) category " ")
                   "\nX-SC-Priority: " (if priority
@@ -842,7 +843,7 @@ the default action of this command is changed to the latter."
                   (mhc-record-subject-as-string record)))))
           (mhc-db-add-exception-rule
            record
-           (or (mhc-current-date)
+           (or (mhc-summary-current-date)
                (mhc-calendar-view-date)))
         (mhc-db-delete-file record))
       (or (and (mhc-summary-buffer-p)
@@ -856,12 +857,18 @@ the default action of this command is changed to the latter."
   (let ((date-list (mapconcat
                     (lambda (day)
                       (mhc-date-format day "%04d%02d%02d" yy mm dd))
-                    (mhc-input-day "Date: " (mhc-current-date))
-                    " ")))
+                    (mhc-input-day "Date: " (mhc-summary-current-date))
+                    " "))
+        (time-list (mhc-time-range-to-string
+                    (mhc-input-time "Time: "
+                                    (mhc-schedule-time-as-string
+                                     (car (mhc-record-schedules
+                                           (mhc-parse-string (mhc-draft-template)))))))))
     (mhc-window-push)
     (mhc-draft-new (mhc-draft-template)
                    `(("x-sc-record-id" . ,(mhc-record-create-id))
                      ("x-sc-sequence"  . 0)
+                     ("x-sc-time"  . ,time-list)
                      ("x-sc-day" . ,date-list)))))
 
 (defun mhc-reuse-copy ()
@@ -871,8 +878,8 @@ the default action of this command is changed to the latter."
         (record (mhc-summary-record)))
     (if (and (stringp file) (file-exists-p file))
         (with-temp-buffer
-	  (mhc-insert-file-contents-as-coding-system
-	   mhc-default-coding-system file)
+          (mhc-insert-file-contents-as-coding-system
+           mhc-default-coding-system file)
           (mhc-header-decode-ewords)
           (mhc-draft-store-template
            (buffer-substring-no-properties (point-min) (point-max)))
@@ -926,54 +933,11 @@ the default action of this command is changed to the latter."
     (message "Specified file(%s) does not exist." file)))
 
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;;
-;; Window stack
-;;
-
-(defvar mhc-window-stack       nil)
-
-(defun mhc-window-push ()
-  (interactive)
-  (setq mhc-window-stack
-        (cons (current-window-configuration) mhc-window-stack)))
-
-(defun mhc-window-pop ()
-  (interactive)
-  (if mhc-window-stack
-      (set-window-configuration (car-safe mhc-window-stack)))
-  (setq mhc-window-stack (cdr-safe mhc-window-stack)))
-
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;
 ;; (Category . (parent-face fg bg))
 ;;
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;; manipulate data from mhc-summary-buffer.
-
-(defconst mhc-summary-day-regex  "\\([^|]+| +\\)?[0-9]+/\\([0-9]+\\)")
-
-(defun mhc-summary-buffer-p (&optional buffer)
-  (if buffer
-      (set-buffer buffer))
-  mhc-summary-buffer-current-date-month)
-
-(defun mhc-current-date ()
-  (when (mhc-summary-buffer-p)
-    (let ((dayinfo (get-text-property (point) 'mhc-dayinfo)))
-      (or (and dayinfo (mhc-day-date dayinfo))
-          (save-excursion
-            (end-of-line)
-            (while (and (not (bobp))
-                        (null dayinfo))
-              (or (setq dayinfo (get-text-property (point) 'mhc-dayinfo))
-                  (forward-char -1)))
-            (and dayinfo (mhc-day-date dayinfo)))))))
-
-(defun mhc-current-date-month ()
-  mhc-summary-buffer-current-date-month)
-
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; misc.
 
